/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 *  USBSink - flashdrive.h
 *
 *  Copyright (C) 2005-2006 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _FLASHDRIVE_H_
#define _FLASHDRIVE_H_

#include <functional>
#include <string>
#include <vector>

#include "exception.h"
#include "uncopyable.h"

#include <tr1/memory>

#include <glibmm/ustring.h>

#include <libhal-storage.h>

namespace USBSink
{

class FlashDrive;
typedef std::tr1::shared_ptr<FlashDrive> SharedFlashDrive;


class FlashDrive : private Uncopyable
{
 public:

  explicit FlashDrive(DBusConnection* dbus_connection,
		      LibHalDrive* drive,
		      LibHalVolume* volume,
                      LibHalDriveBus bus);
  ~FlashDrive();

  bool mount();
  bool umount();

  std::string get_udi() const { return udi_; }
  std::string get_label() const { return label_; }
  LibHalDriveBus get_bus() const { return bus_; }
  std::string get_bus_string() const;
  bool is_mounted() const { return is_mounted_; }
  std::string get_mount_point() const { return mount_point_; }
  std::string get_vendor() const { return vendor_; }
  std::string get_model() const { return model_; }
  std::string get_fs_type() const { return fstype_; }
  unsigned get_size() const { return size_; }
  unsigned get_free_space();
  unsigned file_count() const;
  
  void load_dev_info(LibHalDrive* drive, LibHalVolume* volume);
  void disable();
  
 private:
  
  DBusConnection* dbus_conn_;
  
  std::string udi_; // must be a volume udi
  std::string label_;
  LibHalDriveBus bus_;
  bool is_mounted_;
  std::string mount_point_;
  guint size_;
  std::string vendor_;
  std::string model_;
  std::string fstype_;
  
  Glib::ustring get_mount_error_details(const char* error_name,
					const char* error_message);

  Glib::ustring get_unmount_error_details(const char* error_name,
					  const char* error_message);
};
 
/// operator to use with find().
inline bool
operator==(const SharedFlashDrive& flash, const std::string& udi)
{
  return (flash->get_udi() == udi);
}

inline bool
operator==(const SharedFlashDrive& lhs, const SharedFlashDrive& rhs)
{
  return (lhs->get_udi() == rhs->get_udi());
}

/// functor to be used in find_if() to find a flash by its' label
struct LabelEquals
: public std::binary_function<SharedFlashDrive, Glib::ustring, bool>
{
  bool operator()(const SharedFlashDrive flash,
		  const Glib::ustring& label) const
  {
    return (label == flash->get_label());
  }
};

/**
 * Indicates both mount and unmount errors.
 */
class MountError : public Exception
{
 public:
  MountError(const Glib::ustring& msg) : Exception(msg) {}
};

} // namespace USBSink

#endif
