/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 *  USBSink - halclient.h
 *
 *  Copyright (C) 2005 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _HAL_CLIENT_H_
#define _HAL_CLIENT_H_

#include "exception.h"
#include "flashdrive.h"

#include <memory>
#include <string>
#include <vector>

#include <sigc++/sigc++.h>

#include <libhal-storage.h>

namespace USBSink
{

typedef std::vector<SharedFlashDrive> FlashDriveVector;

class HalClient : private Uncopyable
{
 public:
	
  static HalClient* instance();

  ~HalClient();

  const FlashDriveVector* get_drives() const { return &drives_; }

  // These functions need to be public so that hal C callbacks can access them.
  // They must emit mount and unmount signals,
  // while the class can do added and removed.
  void add_drive(LibHalDrive* drive, LibHalVolume* volume, LibHalDriveBus bus);
  void remove_drive(SharedFlashDrive flash);
  void emit_signal_on_flash(SharedFlashDrive sfd, const char* signal_name);

  SharedFlashDrive find_flash_by_udi(std::string& udi);

  static char* hal_drive_type_to_string(LibHalDriveType drive_type);
  static char* hal_bus_to_string(LibHalDriveBus bus_type);

  sigc::signal<void, SharedFlashDrive> signal_added;
  sigc::signal<void, SharedFlashDrive> signal_removed;
  sigc::signal<void, SharedFlashDrive> signal_mounted;
  sigc::signal<void, SharedFlashDrive> signal_unmounted;

 protected:

  HalClient();

 private:

  bool init_hal();
  bool integrate_hal_into_mainloop(DBusError* error);
  void set_hal_callbacks();
  bool get_drives_at_startup(DBusError* error);
  void disconnect_from_hal();
  
  static std::auto_ptr<HalClient> instance_;

  // There should be one D-BUS connection per application.
  // This is the "original" one that's passed to FlashDrives.
  DBusConnection* dbus_conn_;
  LibHalContext* hal_ctx_;
  
  FlashDriveVector drives_;
};


class HalError : public Exception
{
 public:
  HalError(const Glib::ustring& msg) : Exception(msg) {}
};

} // namespace USBSink

#endif
