/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 *  USBSink - main.cc
 *
 *  Copyright (C) 2006-2007 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "common/nmv-log-stream-utils.h"
#include "halclient.h"
#include "mainwindow.h"
#include "utils.h"

#include <list>

#include <glib/gi18n.h>
#include <glib/gstdio.h>

#include <glibmm/optionentry.h>
#include <glibmm/optiongroup.h>

#include <gtkmm/main.h>

#include <libgnome/gnome-init.h>
#include <libgnomevfsmm/init.h>

#include <config.h>

namespace
{

/**
 * Command-line option parser.
 */
class SinkOptionGroup : public Glib::OptionGroup
{
public:
  SinkOptionGroup();

  bool arg_minimized;
  bool arg_version;
};

SinkOptionGroup::SinkOptionGroup()
:
  Glib::OptionGroup(_("USBSink"),
                    _("USBSink options"),
                    _("Command-line options for usbsink")),
                    arg_minimized(false),
                    arg_version(false)
{
  Glib::OptionEntry entry_minimized, entry_version;

  entry_minimized.set_long_name("minimized");
  entry_minimized.set_short_name('N');
  entry_minimized.set_description(_("Run the program minimized "
                                    "in the system tray"));

  add_entry(entry_minimized, arg_minimized);

  entry_version.set_long_name("version");
  entry_version.set_short_name('V');
  entry_version.set_description(_("The version of this application."));

  add_entry(entry_version, arg_version);
}

/**
 * Checks for and creates if necessary the following directories:
 *
 * ~/.usbsink
 * ~/.usbsink/tasks/
 * ~/.usbsink/file_lists/
 */
void
setup_rc_dir()
{
  using namespace USBSink::Util::Config;

  if (! Glib::file_test(get_rc_dir(), Glib::FILE_TEST_EXISTS))
  {
    if (g_mkdir(get_rc_dir().c_str(), S_IRWXU) < 0)
    {
      g_warning("Could not create %s!", get_rc_dir().c_str());
    }
  }
  if (! Glib::file_test(get_task_dir(), Glib::FILE_TEST_EXISTS))
  {
    if (g_mkdir(get_task_dir().c_str(), S_IRWXU) < 0)
    {
      g_warning("Could not create %s!", get_task_dir().c_str());
    }
  }
  if (! Glib::file_test(get_file_list_dir(), Glib::FILE_TEST_EXISTS))
  {
    if (g_mkdir(get_file_list_dir().c_str(), S_IRWXU) < 0)
    {
      g_warning("Could not create %s!", get_file_list_dir().c_str());
    }
  }

  if (Glib::file_test(get_file_list_dir(), Glib::FILE_TEST_EXISTS))
  {
    create_common_ignore_file_list();
  }
}

} // anonymous namespace

void
on_main_run(Gtk::Window* window, bool minimize)
{
  LOG_FUNCTION_SCOPE_NORMAL_DD;

  if (minimize)
  {
    LOG_DD("Hiding main window on startup");
    window->hide();
  }
}

int
main(int argc, char** argv)
{
  // Make this application use the current locale for _() translation:
  // (USBSINK_LOCALEDIR is defined in the Makefile.am)
  bindtextdomain(GETTEXT_PACKAGE, USBSINK_LOCALEDIR);
  bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
  textdomain(GETTEXT_PACKAGE);

  try
  {
    Glib::OptionContext option_context;
    SinkOptionGroup option_group;

    option_context.set_main_group(option_group);

    // Initialize gnome_program, so that we can use gnome_help_display()
    gnome_program_init(PACKAGE_NAME, PACKAGE_VERSION,
                       LIBGNOME_MODULE,
                       argc, argv,
                       GNOME_PARAM_HUMAN_READABLE_NAME, _("USBSink"),
                       GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);

    setup_rc_dir();

    Gnome::Vfs::init();

    Gtk::Main kit(argc, argv, option_context);

    if (option_group.arg_version)
    {
      g_printf("%s %s\n", PACKAGE_NAME, PACKAGE_VERSION);
      return 0;
    }

    bool run_minimized = option_group.arg_minimized;

    USBSink::UI::MainWindow window(run_minimized);

    Gtk::Main::signal_run().connect(
      sigc::bind(sigc::ptr_fun(on_main_run),
                 window.get_window(), run_minimized));

    window.get_window()->show_all();

    Gtk::Main::run();
  }
  catch (const Glib::OptionError& ex)
  {
    g_printf("%s\n", ex.what().c_str());
  }
  catch (const Glib::Error& ex)
  {
    g_error("unhandled exception: %s", ex.what().c_str());
  }
  catch (const std::exception ex)
  {
    g_error("unhandled exception: %s", ex.what());
  }
  catch (...)
  {
    g_error("unhandled exception [type unknown]");
  }
  
  return 0;
}
