/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 *  USBSink - newtaskdialog.cc
 *
 *  Copyright (C) 2005-2006 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "addfilesdialog.h"
#include "blackfilelist.h"
#include "filelisteditor.h"
#include "filelistregistry.h"
#include "flashdrive.h"
#include "globalstrings.h"
#include "halclient.h"
#include "newtaskdialog.h"
#include "task.h"
#include "taskmanager.h"
#include "utils.h"
#include "whitefilelist.h"

#include <functional>
#include <list>
#include <sstream>
#include <string>
#include <vector>

#include <glib/gi18n.h>

#include <glibmm/refptr.h>
#include <glibmm/ustring.h>

#include <gtkmm.h>

#include <libglademm/xml.h>

#include <sigc++/sigc++.h>

#include <libgnomevfsmm.h>

using std::list;
using std::vector;

namespace USBSink
{

typedef vector<Glib::ustring> ustring_vector;
typedef vector<Glib::ustring>::iterator ustring_vector_iter;

typedef list< Glib::RefPtr<Gnome::Vfs::Uri> > uri_list;
typedef list< Glib::RefPtr<Gnome::Vfs::Uri> >::iterator uri_list_iter;

typedef Gtk::TreeModel::Children tree_children;

typedef std::tr1::shared_ptr<TaskHandling::Task> SharedTask;

//

namespace UI
{

class TaskNameInUseException : public std::exception
{ /* no implementation */ };

void
NewTaskDialog::AddToComboBox::operator()(const SharedFlashDrive flash,
                                         const NewTaskDialog* dlg) const
{
  Gtk::TreeModel::Row row = *(dlg->treemodel_drives_->append());
  row[dlg->columns_removables_.col_udi] = flash->get_udi();
  row[dlg->columns_removables_.col_label] = flash->get_label();
}

NewTaskDialog::NewTaskDialog(Gtk::Window& parent)
:
  dialog_                      (),
  entry_task_name_             (0),
  combobox_target_drive_       (0),
  combobox_data_flow_          (0),
  combobox_file_list_type_     (0),
  textview_file_list_          (0),
  button_file_list_editor_     (0),
  combobox_start_              (0),
  spinbutton_timeout_          (0),
  toolbar_                     (0),
  toolbutton_add_              (0),
  toolbutton_remove_           (0),
  toolbutton_clear_            (0),
  toolbutton_import_           (0),
  treeview_file_list_          (0),
  entry_target_dir_            (0),
  button_browse_dir_           (0)
{
  load_xml();
  dialog_->set_focus(*entry_task_name_);
  dialog_->set_transient_for(parent);
  connect_signals();

  combobox_data_flow_->set_active(0);
  populate_drive_combobox();
  combobox_start_->set_active(0);

  combobox_file_list_type_->set_active(0);
  textview_buffer_ = Gtk::TextBuffer::create();
//  textview_buffer_->set_text("<common>\n");
  textview_file_list_->set_buffer(textview_buffer_);

  setup_file_list_layout();

  dnd_targets_.push_back(Gtk::TargetEntry("text/uri-list"));
  treeview_file_list_->drag_dest_set(dnd_targets_);
}

NewTaskDialog::~NewTaskDialog()
{}

void
NewTaskDialog::load_xml()
{
  using Gnome::Glade::Xml;

  try
  {
    const Glib::RefPtr<Xml> xml = Xml::create(glade_newtaskdialog_filename);

    Gtk::Dialog* dialog_ptr = 0;
    dialog_.reset(xml->get_widget("newtaskdialog", dialog_ptr));

    xml->get_widget("entry_task_name", entry_task_name_);
    xml->get_widget("combobox_target_drive", combobox_target_drive_);
    xml->get_widget("combobox_data_flow", combobox_data_flow_);
    xml->get_widget("combobox_file_list_type", combobox_file_list_type_);
    xml->get_widget("textview_file_list", textview_file_list_);
    xml->get_widget("button_file_list_editor", button_file_list_editor_);
    xml->get_widget("combobox_start", combobox_start_);
    xml->get_widget("spinbutton_timeout", spinbutton_timeout_);

    xml->get_widget("toolbar", toolbar_);
    xml->get_widget("toolbutton_add", toolbutton_add_);
    xml->get_widget("toolbutton_remove", toolbutton_remove_);
    xml->get_widget("toolbutton_clear", toolbutton_clear_);
    xml->get_widget("toolbutton_import", toolbutton_import_);
    xml->get_widget("treeview_file_list", treeview_file_list_);

    xml->get_widget("entry_target_dir", entry_target_dir_);
    xml->get_widget("button_browse_dir", button_browse_dir_);
  }
  catch (const Gnome::Glade::XmlError& xml_err)
  {
    g_error("%s", xml_err.what().c_str());
  }
}

void
NewTaskDialog::connect_signals()
{
  dialog_->signal_response().connect(
    sigc::mem_fun(*this, &NewTaskDialog::on_response));

  button_file_list_editor_->signal_clicked().connect(
    sigc::mem_fun(*this, &NewTaskDialog::on_file_list_editor_clicked));

  toolbutton_add_->signal_clicked().connect(
    sigc::mem_fun(*this, &NewTaskDialog::on_toolbutton_add));

  toolbutton_remove_->signal_clicked().connect(
    sigc::mem_fun(*this, &NewTaskDialog::on_toolbutton_remove));

  toolbutton_clear_->signal_clicked().connect(
    sigc::mem_fun(*this, &NewTaskDialog::on_toolbutton_clear));

  toolbutton_import_->signal_clicked().connect(
    sigc::mem_fun(*this, &NewTaskDialog::on_toolbutton_import));
  
  button_browse_dir_->signal_clicked().connect(
    sigc::mem_fun(*this, &NewTaskDialog::on_browse_dir));

  treeview_file_list_->signal_drag_data_received().connect(
    sigc::mem_fun(*this, &NewTaskDialog::on_treeview_drag_data_received));
}

/// Passes the request to the main window, who is in charge for all dialogs.
void
NewTaskDialog::on_file_list_editor_clicked()
{
  try
  {
    check_is_task_name_duplicate();
    signal_file_list_editor_request.emit(entry_task_name_->get_text());
  }
  catch (const TaskNameInUseException& ex)
  {
    handle_duplicate_task_name(entry_task_name_->get_text());
  }
}

void
NewTaskDialog::on_toolbutton_add()
{
  if (addfiles_dialog_.get())
  {
     // The dialog is already open, and the user asked for it once again;
     // so move it where the user can see it.
     // Btw present() calls Widget::show() if a window is hidden.
     addfiles_dialog_->get_dialog()->present();
  }
  else
  {
    Glib::ustring mnt_point("/media/");

    if (combobox_data_flow_->get_active_row_number()
	== TaskHandling::DATA_FLOW_REMOVABLE_TO_LOCAL)
    {
      tree_children::iterator it = combobox_target_drive_->get_active();

      if (it)
      {      
	try
	{
	  HalClient* hc = HalClient::instance();
	  std::string s = (*it)[columns_removables_.col_udi];
	  SharedFlashDrive flash = hc->find_flash_by_udi(s);
	  
	  if (flash->is_mounted())
	  {
	    mnt_point = flash->get_mount_point();
	  }
	  else
	  {
	    g_warning("selected removable drive %s in not mounted",
		      flash->get_label().c_str()); 
	  }
	}
	catch (HalError& err)
	{
	  g_warning("HAL exception: %s", err.what().c_str());
	}
      }
    }

    bool src_files_from_removable =
      (combobox_data_flow_->get_active_row_number() ==
       TaskHandling::DATA_FLOW_REMOVABLE_TO_LOCAL);

    std::auto_ptr<AddFilesDialog> dialog(
      new AddFilesDialog(*dialog_, src_files_from_removable, mnt_point));
    
    dialog->get_dialog()->signal_hide().connect(
      sigc::mem_fun(*this, &NewTaskDialog::on_addfilesdialog_hide));

    dialog->signal_file_uris_selected.connect(
      sigc::mem_fun(*this, &NewTaskDialog::on_addfilesdialog_files_selected));

    dialog->get_dialog()->show();
    
    addfiles_dialog_ = dialog;
  }
}

void
NewTaskDialog::on_toolbutton_remove()
{
  typedef vector<Gtk::TreeModel::Path> path_vector;
  typedef vector<Gtk::TreeIter> tree_iter_vector;

  path_vector sel_rows = file_list_selection_->get_selected_rows();
  tree_iter_vector tree_iters;
	
  path_vector::iterator p_it(sel_rows.begin());
  path_vector::iterator p_end(sel_rows.end());

  for (; p_it != p_end; ++p_it)
  {
    tree_iters.push_back(treemodel_files_->get_iter(*p_it));
  }

  tree_iter_vector::iterator t_it(tree_iters.begin());
  tree_iter_vector::iterator t_end(tree_iters.end());

  for (; t_it != t_end; ++t_it)
  {
    treemodel_files_->erase(*t_it);
  }
}

void
NewTaskDialog::on_toolbutton_clear()
{
  treemodel_files_->clear();
}

void
NewTaskDialog::on_toolbutton_import()
{
  Gtk::FileChooserDialog dialog(*(this->get_dialog()),
				_("Select a task file"),
				Gtk::FILE_CHOOSER_ACTION_OPEN);

  dialog.set_current_folder(Util::Config::get_task_dir());

  // Add response buttons.
  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

  int result = dialog.run();

  if (result == Gtk::RESPONSE_OK)
  {
    TaskHandling::Manager* tm = TaskHandling::Manager::instance();
    SharedTask task(tm->get_task(dialog.get_filename()));

    if (task.get())
    {
      uri_list src_files(task->get_source_files());
      on_addfilesdialog_files_selected(src_files);
    }
  }
}

void
NewTaskDialog::on_addfilesdialog_hide()
{
  // Transfer ownership and let the new auto_ptr's destructor do the delete.
  const std::auto_ptr<AddFilesDialog> temp(addfiles_dialog_);
}

void
NewTaskDialog::on_browse_dir()
{
  Gtk::FileChooserDialog dialog(*(this->get_dialog()),
				_("Select a directory"),
				Gtk::FILE_CHOOSER_ACTION_SELECT_FOLDER);

  if (combobox_data_flow_->get_active_row_number() ==
      static_cast<int>(TaskHandling::DATA_FLOW_LOCAL_TO_REMOVABLE))
  {
    Glib::ustring mnt_point("/media/");
    tree_children::iterator it = combobox_target_drive_->get_active();

    if (it)
    {
      try
      {
	HalClient* hc = HalClient::instance();
	std::string udi = (*it)[columns_removables_.col_udi];
	SharedFlashDrive flash = hc->find_flash_by_udi(udi);
	
	if (flash->is_mounted())
	{
	  mnt_point = flash->get_mount_point();
	}
	else
	{
	  g_warning("selected removable drive %s in not mounted",
		    flash->get_label().c_str());
	}
      }
      catch (HalError& err)
      {
	g_warning("HAL exception: %s", err.what().c_str());
      }
    }

    dialog.set_current_folder(mnt_point);
  }
  else if (combobox_data_flow_->get_active_row_number() ==
	   static_cast<int>(TaskHandling::DATA_FLOW_REMOVABLE_TO_LOCAL))
  {
    dialog.set_current_folder(Glib::get_home_dir());
  }

  // Add response buttons.
  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog.add_button(Gtk::Stock::OK, Gtk::RESPONSE_OK);

  int result = dialog.run();

  if (result == Gtk::RESPONSE_OK)
  {
    entry_target_dir_->set_text(dialog.get_filename());
  }
}

void
NewTaskDialog::populate_drive_combobox()
{
  // create the tree model
  treemodel_drives_ = Gtk::ListStore::create(columns_removables_);
  combobox_target_drive_->set_model(treemodel_drives_);

  try
  {
    HalClient* hc = HalClient::instance();
    const FlashDriveVector* drives = hc->get_drives();

    if (! drives->empty())
    {
      std::for_each(drives->begin(),
		    drives->end(),
		    std::bind2nd(AddToComboBox(), this));
      
      combobox_target_drive_->pack_start(columns_removables_.col_label);
      combobox_target_drive_->set_active(0); // default is the first item
    }
  }
  catch (HalError& err)
  {
    g_warning("HAL exception while trying to populate drive combobox: %s",
	      err.what().c_str());
  }
}

void
NewTaskDialog::setup_file_list_layout()
{
  // create the tree model
  treemodel_files_ = Gtk::ListStore::create(columns_files_);
  treeview_file_list_->set_model(treemodel_files_);

  // set up the treeview selection object
  file_list_selection_ = treeview_file_list_->get_selection();
  file_list_selection_->set_mode(Gtk::SELECTION_MULTIPLE);

  // append column headers
  // the name column will have an associated cellrenderer
  // because we want, later in the EDT, to display non-existing URIs in italic
  Gtk::CellRendererText* renderer = Gtk::manage(new Gtk::CellRendererText());
  Gtk::TreeViewColumn* name_column =
    Gtk::manage(new Gtk::TreeViewColumn(_("Name"), *renderer));

  name_column->add_attribute(renderer->property_markup(),
                             columns_files_.col_name);

  treeview_file_list_->append_column(*name_column);
  treeview_file_list_->append_column(_("Modified"), columns_files_.col_modified);
  treeview_file_list_->append_column(_("MIME type"),columns_files_.col_mime_type);
  treeview_file_list_->append_column(_("Full path"),columns_files_.col_full_path);
}

bool
NewTaskDialog::file_is_in_list(const Glib::ustring& uri)
{
  tree_children children = treemodel_files_->children();
  
  tree_children::iterator it(children.begin());
  tree_children::iterator end(children.end());

  for (; it != end; ++it)
  {
    Glib::ustring row_full_path((*it)[columns_files_.col_full_path]);
    if (row_full_path == uri)
    {
      return true;
    }
  }
  return false;
}

void
NewTaskDialog::add_uri_to_treeview(const Glib::RefPtr<Gnome::Vfs::Uri>& uri)
{
  if (! file_is_in_list(uri->get_path()))
  {
    try
    {
      Glib::RefPtr<Gnome::Vfs::FileInfo> fi =
        uri->get_file_info(Gnome::Vfs::FILE_INFO_GET_MIME_TYPE);
      
      Gtk::TreeModel::Row row = *(treemodel_files_->append());
      
      row[columns_files_.col_name] = uri->extract_short_name();
      
      row[columns_files_.col_modified] =
        Util::Time::get_modification_date_string(fi->get_modification_time());
      
      row[columns_files_.col_mime_type] = fi->get_mime_type();
    
      if (uri->is_local())
      {
        row[columns_files_.col_full_path] = uri->get_path();
      }
      else
      {
        // for instance, it would be smb:://hostname/dir/file
        row[columns_files_.col_full_path] = uri->to_string();
      }
    }
    catch (Gnome::Vfs::exception& )
    {
      // uri doesn't exist, display only the short name in italic
      Gtk::TreeModel::Row row = *(treemodel_files_->append());
      row[columns_files_.col_name] =
        "<i>" + uri->extract_short_name() + "</i>";
    }
  }
}

void
NewTaskDialog::on_addfilesdialog_files_selected(uri_list& uris)
{
  uri_list_iter it(uris.begin());
  uri_list_iter end(uris.end());
  
  for (; it != end; ++it)
  {
    if (*it) // there's going to be a number of nulls in the list
    {
      add_uri_to_treeview(*it);
    }
  }
}

void
NewTaskDialog::on_treeview_drag_data_received(
                           const Glib::RefPtr<Gdk::DragContext>& /* context */,
                           int /* x */, int /* y */,
                           const Gtk::SelectionData& selection_data,
                           guint /* info */, guint /* time */)
{
  if (selection_data.get_data())
  {
    ustring_vector targets = selection_data.get_uris();

    if (targets.empty())
      return;

    for (ustring_vector_iter it(targets.begin()); it != targets.end(); ++it)
    {
      try
      {
        Glib::RefPtr<Gnome::Vfs::Uri> uri = Gnome::Vfs::Uri::create(*it);
        add_uri_to_treeview(uri);
      }
      catch (const Gnome::Vfs::exception& ex)
      {
        g_warning("Error while getting file info for path %s: %s",
                  it->c_str(),
                  ex.what().c_str());
      }
    }
  }
}

uri_list
NewTaskDialog::get_source_files()
{
  uri_list src_files;

  tree_children children = treemodel_files_->children();
  uri_list uris(children.size());
  
  tree_children::iterator it(children.begin());
  tree_children::iterator end(children.end());
		
  for (; it != end; ++it)
  {
    try
    {
      Glib::RefPtr<Gnome::Vfs::Uri> uri =
	Gnome::Vfs::Uri::create((*it)[columns_files_.col_full_path]);
      
      if (uri->uri_exists())
      {
	src_files.push_back(uri);
      }
    }
    catch (const Gnome::Vfs::exception& ex)
    {
      Glib::ustring path((*it)[columns_files_.col_full_path]);
      g_warning("Error while getting file info for path %s: %s",
		path.c_str(),
		ex.what().c_str());
    }
  }

  return src_files;
}

/**
 * Checks user input.
 * Throws InputError with a list of things user should correct,
 * or IgnoreFilesException if the all input is ok but some ignore file lists
 * do not exist and user chose to create them in a TODO dialog or to go back
 * to *this.
 */
void
NewTaskDialog::check_input()
{
  Glib::ustring message;

  if (entry_task_name_->get_text() == Glib::ustring())
  {
    message += _("Task name");
    message += "\n";
  }

  if (combobox_target_drive_->get_active_row_number() == -1)
  {
    message += _("Removable drive selection");
    message += "\n";
  }

  // there's no need to force a file list

  if (treemodel_files_->children().empty())
  {
    message += _("Source files");
    message += "\n";
  }

  if (entry_target_dir_->get_text() == Glib::ustring())
  {
    message += _("Target directory");
    message += "\n";
  }

  if (message != Glib::ustring())
  {
    Glib::ustring err_msg =
      Util::i18n_printf(_("Please check the following:\n\n%s"), message.c_str());
    throw InputError(err_msg);
  }
}

/**
 * @throw TaskNameInUseException
 */
void
NewTaskDialog::check_is_task_name_duplicate()
{
  TaskHandling::Manager* task_manager = TaskHandling::Manager::instance();

  if (task_manager->task_exists(entry_task_name_->get_text()))
  {
    throw TaskNameInUseException();
  }
}

void
NewTaskDialog::handle_duplicate_task_name(const Glib::ustring& name)
{
  Glib::ustring message(
    Util::i18n_printf(_("Name '%s' is already in use."), name.c_str()));
  Gtk::MessageDialog err_dialog(*(dialog_->get_transient_for()),
                                message, false, Gtk::MESSAGE_ERROR,
                                Gtk::BUTTONS_OK, true);
  Glib::ustring sugg = _("Please choose another name.");
  err_dialog.set_secondary_text(sugg);
  err_dialog.run();
}

/**
 * @throw TaskNameInUseException
 */
SharedTask
NewTaskDialog::create_task_from_input()
{
  check_is_task_name_duplicate();

  TaskHandling::Manager* task_manager = TaskHandling::Manager::instance();

  // id, name
  int id = task_manager->get_new_task_id();
  Glib::ustring name = entry_task_name_->get_text();

  // drive udi
  tree_children::iterator drive_iter = combobox_target_drive_->get_active();
  std::string drive_udi = (*drive_iter)[columns_removables_.col_udi];

  // data flow
  TaskHandling::DataFlow data_flow =
    static_cast<TaskHandling::DataFlow>(
      combobox_data_flow_->get_active_row_number());

  // list
  TaskHandling::FileListRegistry* registry =
    TaskHandling::FileListRegistry::instance();

  TaskHandling::file_list_factory_from_buffer factory;

  if (combobox_file_list_type_->get_active_row_number() == 0)
  {
    factory = &TaskHandling::BlackFileList::create;
  }
  else if (combobox_file_list_type_->get_active_row_number() == 1)
  {
    factory = &TaskHandling::WhiteFileList::create;
  }
  else
  {
    factory = &TaskHandling::BlackFileList::create;
  }

  TaskHandling::SharedItem file_list =
    registry->register_new_file_list(
      name, textview_buffer_, factory);

  // when to start
  TaskHandling::StartTimeOption when_to_start =
    static_cast<TaskHandling::StartTimeOption>(
      combobox_start_->get_active_row_number());

  // timeout
  int timeout = spinbutton_timeout_->get_value_as_int();
  timeout *= ms_in_min; // convert minutes to miliseconds

  // source files
  std::list< Glib::RefPtr<Gnome::Vfs::Uri> > src_files = get_source_files();
  
  // target dir
  Glib::ustring target_dir = entry_target_dir_->get_text();

  SharedTask task(
    new TaskHandling::Task(id, name, drive_udi, data_flow, file_list,
             when_to_start, timeout, src_files, target_dir));

  return task;
}

void
NewTaskDialog::register_task()
{
  try
  {
    check_input();

    SharedTask task = create_task_from_input();

    TaskHandling::Manager* manager = TaskHandling::Manager::instance();
    manager->add_task(task);
    
    dialog_->hide();
  }
  catch (InputError& ex)
  {
    Glib::ustring message(_("Insuficient input."));
    Gtk::MessageDialog err_dialog(*(dialog_->get_transient_for()),
                                  message, false, Gtk::MESSAGE_ERROR,
                                  Gtk::BUTTONS_OK, true);
    Glib::ustring err_details = ex.what();
    err_dialog.set_secondary_text(err_details);
    err_dialog.run();
  }
  catch (const TaskNameInUseException& ex)
  {
    handle_duplicate_task_name(entry_task_name_->get_text());
  }
  catch (Glib::FileError& ex) // from save_task()'s iochannel
  {
    Gtk::MessageDialog err_dialog(*(dialog_->get_transient_for()),
                                  _("Could not save task to a file"),
                                  false, Gtk::MESSAGE_ERROR,
                                  Gtk::BUTTONS_OK, true);
    
    Glib::ustring err_details = Glib::strerror(ex.code());
    err_dialog.set_secondary_text(err_details);
    err_dialog.run();
  }
}

void
NewTaskDialog::on_response(int response_id)
{
  if (response_id == Gtk::RESPONSE_OK)
  {
    register_task();
  }
  else
  {
    dialog_->hide();
  }
}

} // namespace UI

} // namespace USBSink
