/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 *  USBSink - task.h
 *
 *  Copyright (C) 2006-2007 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _TASK_H_
#define _TASK_H_

#include "enums.h"
#include "fileentry.h"
#include "filewatcher.h"
#include "flashdrive.h"
#include "halclient.h"
#include "logger.h"
#include "filelist.h"

#include <list>
#include <string>
#include <tr1/memory>
#include <vector>

#include <glibmm/refptr.h>
#include <glibmm/ustring.h>

#include <libgnomevfs/gnome-vfs-async-ops.h>

#include <libgnomevfsmm/transfer-progress.h>
#include <libgnomevfsmm/uri.h>

namespace USBSink
{

namespace Transfer { class Statistics; }


namespace TaskHandling
{

class Task;
typedef std::tr1::shared_ptr<Task> SharedTask;


enum DataFlow
{
  DATA_FLOW_REMOVABLE_TO_LOCAL,
  DATA_FLOW_LOCAL_TO_REMOVABLE
};

enum StartTimeOption
{
  START_TIME_OPTION_NOW,
  START_TIME_OPTION_LATER
};


class Task : public sigc::trackable
{
 public:

  explicit Task(const int id,
                const Glib::ustring& task_name,
                const std::string& drive_udi,
                const DataFlow data_flow,
                const SharedItem& file_list,
                const StartTimeOption when_to_start,
                const int timeout,
                const std::list< Glib::RefPtr<Gnome::Vfs::Uri> >& src_files,
                const Glib::ustring& target_dir);
  ~Task();

  int                   get_id() const;
  Glib::ustring         get_name() const;
  std::string           get_filename() const;
  std::string           get_drive_udi() const;
  SharedFlashDrive      get_drive() const;
  DataFlow              get_data_flow() const;
  SharedFileList        get_file_list() const;
  StartTimeOption       get_start_time_option() const;
  int                   get_timeout() const;
  std::list< Glib::RefPtr<Gnome::Vfs::Uri> > get_source_files() const;
  Glib::ustring         get_target_dir() const;
  Transfer::Logger*     get_logger();
  Transfer::Statistics& get_statistics();
  Glib::ustring         get_printed_bytes_transferred() const;

  void set_name(const Glib::ustring& new_name);
  void set_drive(const SharedFlashDrive& new_drive);
  void set_file_list(SharedFileList& new_list);
  void set_source_files(
    const std::list< Glib::RefPtr<Gnome::Vfs::Uri> >& files);
  void set_target_dir(const Glib::ustring& new_dir);

  void set_timeout(const int new_interval);
  
  void start(bool mounted = false);
  void stop(const Glib::ustring& note, bool unmounted = false);

  bool is_running() const;

  void inc_transfer_count();
  void dec_transfer_count();

  sigc::signal<void, Task*> signal_started;
  sigc::signal<void, Task*, Glib::ustring> signal_stopped;
  sigc::signal<void, Task*> signal_begin_sync;
  sigc::signal<void, Task*> signal_end_sync;

 protected:

  SharedFileEntry find_file_entry(const Glib::ustring& path);

  void add_source_file(const Glib::RefPtr<Gnome::Vfs::Uri>& uri,
                       Inclusion type = INCLUSION_EXPLICIT);

  void on_monitor(const Glib::ustring& monitor_uri,
		  const Glib::ustring& info_uri,
		  FileStatus status);

  static gint async_transfer_callback(GnomeVFSAsyncHandle* handle,
                                      GnomeVFSXferProgressInfo* info,
                                      gpointer callback_data);

  bool on_timeout();
  void on_sync_done();
  
private:

  int id_;

  Glib::ustring name_;
  std::string file_name_;

  std::string drive_udi_;
  SharedFlashDrive target_drive_;

  DataFlow data_flow_;

  SharedFileList file_list_;

  StartTimeOption when_to_start_;

  sigc::connection timeout_connection_;
  
  int timeout_interval_;

  std::list<SharedFileEntry> src_files_;
  Glib::ustring target_dir_;

  bool is_running_;

  Transfer::Logger* logger_;

  Transfer::Statistics* stats_;
  
  int pending_xfer_count_;
};


struct less_id : public std::binary_function<SharedTask, SharedTask, bool>
{
  bool operator()(const SharedTask& x, const SharedTask& y) const
  {
    return x->get_id() < y->get_id();
  }
};

// Task get_* functions

inline int
Task::get_id() const
{
  return id_;
}

inline Glib::ustring
Task::get_name() const
{
  return name_;
}

inline std::string
Task::get_filename() const
{
  return file_name_;
}

inline std::string
Task::get_drive_udi() const
{
  return drive_udi_;
}

inline SharedFlashDrive
Task::get_drive() const
{
  return target_drive_;
}

inline DataFlow
Task::get_data_flow() const
{
  return data_flow_;
}

inline SharedFileList
Task::get_file_list() const
{
  return file_list_;
}

inline int
Task::get_timeout() const
{
  return timeout_interval_;
}

inline StartTimeOption
Task::get_start_time_option() const
{
  return when_to_start_;
}

inline Glib::ustring
Task::get_target_dir() const
{
  return target_dir_;
}

inline bool
Task::is_running() const
{
  return is_running_;
}

inline Transfer::Logger*
Task::get_logger()
{
  return logger_;
}

inline Transfer::Statistics&
Task::get_statistics()
{
  return *stats_;
}

// Task set_* functions

inline void
Task::set_drive(const SharedFlashDrive& new_drive)
{
  target_drive_ = new_drive;
}

inline void
Task::set_target_dir(const Glib::ustring& new_dir)
{
  target_dir_ = new_dir;
}

inline void
Task::inc_transfer_count()
{
  pending_xfer_count_++;
}

inline void
Task::dec_transfer_count()
{
  pending_xfer_count_--;
}

} // namespace TaskHandling

} // namespace USBSink

#endif // _TASK_H_
