/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 *  USBSink - taskmanager.cc
 *
 *  Copyright (C) 2006 Marko Anastasov
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "filelist.h"
#include "filelistregistry.h"
#include "mainwindow.h"
#include "task.h"
#include "taskmanager.h"
#include "taskfileparser.h"
#include "utils.h"

#include <algorithm>
#include <cstdio>
#include <fstream>
#include <string>

#include <glibmm.h>

namespace USBSink
{

namespace TaskHandling
{

using std::auto_ptr;
using std::list;

typedef list<SharedTask>::iterator task_iter;
typedef list<SharedTask>::const_iterator task_const_iter;

auto_ptr<Manager> Manager::instance_;

/**
 * Loads and runs all tasks in ~/.config/usbsink/tasks.
 */
void
Manager::load_tasks()
{
  list<std::string> entries;

  try
  {
    entries = Util::File::get_files_in_dir(Util::Config::get_task_dir());
    
    list<std::string>::iterator it(entries.begin());
    list<std::string>::iterator end(entries.end());
    
    FileParser parser;
  
    sigc::connection parser_connection = parser.signal_task_loaded.connect(
      sigc::mem_fun(*this, &Manager::on_task_loaded));

    for ( ; it != end; ++it)
    {
      Glib::Markup::ParseContext context(parser);
      
      Glib::RefPtr<Glib::IOChannel> channel =
	Glib::IOChannel::create_from_file(*it, "r");

      Glib::ustring buf;

      try
      {
	while (channel->read_line(buf) == Glib::IO_STATUS_NORMAL)
	  context.parse(buf);

	context.end_parse();
      }
      catch (Glib::MarkupError& ex)
      {
	Glib::ustring error_details(markup_error_code_to_string(ex.code()));
	g_warning("Error while parsing task file: %s", error_details.c_str());
	//TODO: show this in a dialog somewhere
      }
    }

    parser_connection.disconnect();
  }
  catch (Glib::FileError& ex)
  {
    g_critical("Could not load task files: %s", ex.what().c_str());
    //TODO: propagate to UI
  }
}

Manager::Manager()
{
  HalClient* hc = HalClient::instance();

  hc->signal_mounted.connect(
	sigc::mem_fun(*this, &Manager::on_drive_mounted));

  hc->signal_unmounted.connect(
    sigc::mem_fun(*this, &Manager::on_drive_unmounted));

  FileListRegistry* list_registry = FileListRegistry::instance();

  list_registry->get_signal_file_list_reset().connect(
    sigc::mem_fun(*this, &Manager::on_file_list_reset));
}

Manager::~Manager()
{
  tasks_.clear();
}

Manager*
Manager::instance()
{
  if (! instance_.get())
  {
    instance_.reset(new Manager);
  }
  return instance_.get();
}

void
Manager::init()
{
  instance()->load_tasks();
}

int
Manager::get_new_task_id() const
{
  int res = 0;

  if (! tasks_.empty())
  {
    list<SharedTask>::const_iterator it =
      max_element(tasks_.begin(), tasks_.end(), less_id());
    res = (*it)->get_id();
  }

  return ++res;
}

SharedTask
Manager::get_task(const int id)
{
  // TODO: check if id exists, notify

  Task* t = 0;
  SharedTask task(t);

  task_iter it(tasks_.begin());
  task_iter end(tasks_.end());

  for ( ; it != end; ++it)
  {
    if ((*it)->get_id() == id)
    {
      task = *it;
      break;
    }
  }

  return task;
}

SharedTask
Manager::get_task(const Glib::ustring& filename)
{
  // TODO: check if filename exists, notify

  Task* t = 0;
  SharedTask task(t);

  task_iter it(tasks_.begin());
  task_iter end(tasks_.end());

  for ( ; it != end; ++it)
  {
    if ((*it)->get_filename() == filename)
    {
      task = *it;
      break;
    }
  }

  return task;
}

const Glib::ustring
Manager::get_task_name(int id) const
{
  Glib::ustring name;

  list<SharedTask>::const_iterator it(tasks_.begin());
  list<SharedTask>::const_iterator end(tasks_.end());

  for ( ; it != end; ++it)
  {
    if ((*it)->get_id() == id)
    {
      name = (*it)->get_name();
    }
  }

  return name;
}

/**
 * @throw Glib::FileError
 */
void
Manager::add_task(SharedTask& task, bool save, bool start)
{
  if ((task->get_drive().get() != 0) &&
      task->get_drive()->is_mounted() &&
      start)
  {
    task->start();
  }

  tasks_.push_back(task);
  signal_task_added_.emit(task);

  if (save)
  {
    FileParser tf_parser;
    tf_parser.save_task(task);
  }
}

void
Manager::remove_task(int task_id)
{
  SharedTask task(get_task(task_id));
  task->stop(Glib::ustring());
  signal_task_removed_.emit(task_id);

  // delete the task file
  if (remove(task->get_filename().c_str()) == -1)
    g_warning("Error deleting task file %s", task->get_filename().c_str());

  // and the corresponding list
  FileListRegistry* registry = FileListRegistry::instance();
  registry->delete_file_list(task->get_name());

  tasks_.erase(std::remove(tasks_.begin(), tasks_.end(), task));
}

void
Manager::on_task_loaded(SharedTask& task, bool start)
{
  add_task(task, false, start);
}

list<SharedTask>
Manager::get_tasks_on_drive(const std::string& udi)
{
  list<SharedTask> tasks;

  task_iter it(tasks_.begin());
  task_iter end(tasks_.end());

  for ( ; it != end; ++it)
  {
    if ((*it)->get_drive_udi() == udi)
    {
      tasks.push_back(*it);
    }
  }

  return tasks;
}

void
Manager::on_drive_mounted(const SharedFlashDrive& drive)
{
  // Start all tasks associated with the mounted drive;
  // also it is possible that the drive has just been inserted for the
  // first time, so we must update the tasks' SharedFlashDrive from
  // null to something.

  list<SharedTask> tasks = get_tasks_on_drive(drive->get_udi());

  task_iter it(tasks_.begin());
  task_iter end(tasks_.end());

  for ( ; it != end; ++it)
  {
    (*it)->set_drive(drive);

    if ((*it)->get_start_time_option() == START_TIME_OPTION_NOW)
    {
      (*it)->start(true);
    }
  }
}

void
Manager::on_drive_unmounted(const SharedFlashDrive& drive)
{
  list<SharedTask> tasks = get_tasks_on_drive(drive->get_udi());

  task_iter it(tasks_.begin());
  task_iter end(tasks_.end());

  for ( ; it != end; ++it)
  {
    (*it)->stop("unmounted", true);
    // the text will be displayed in the main window's treeview
  }
}

void
Manager::on_application_exit(USBSink::UI::MainWindow* main_window)
{
  if (main_window->get_hide_in_tray())
  {
    // Keep on running, the application window has only been minimized
    // to system tray.
    return;
  }

  // Remember last state.
  FileParser tf_parser;

  task_iter it(tasks_.begin());
  task_iter end(tasks_.end());

  for ( ; it != end; ++it)
  {
    tf_parser.save_task(*it);
  }
}

/**
 * Signal handler for a signal got from
 * FileListRegistry::get_file_list_reset_signal().
 *
 * Attempts to find a task which "owns" this list, and reset its file list.
 */
void
Manager::on_file_list_reset(SharedFileList& list)
{
  task_iter it(tasks_.begin());
  task_iter end(tasks_.end());

  for ( ; it != end; ++it)
  {
    if ((*it)->get_name() == list->get_name())
    {
      (*it)->set_file_list(list);

      // there's one and only one such task
      break;
    }
  }
}

bool
Manager::task_exists(const Glib::ustring& name) const
{
  bool result = false;

  task_const_iter it(tasks_.begin());
  task_const_iter end(tasks_.end());

  for ( ; it != end; ++it)
  {
    if ((*it)->get_name() == name)
    {
      result = true;
      break;
    }
  }

  return result;
}

sigc::signal< void, const SharedTask& >&
Manager::get_signal_task_added()
{
  return signal_task_added_;
}

sigc::signal< void, int >&
Manager::get_signal_task_removed()
{
  return signal_task_removed_;
}

int
Manager::get_active_task_count() const
{
  int res = 0;
  task_const_iter it(tasks_.begin());
  task_const_iter end(tasks_.end());

  for ( ; it != end; ++it)
    if ((*it)->is_running())
      res++;

  return res;
}

} // namespace TaskHandling

} // namespace USBSink
